@@ -1,5 +1,5 @@
 package Git::Raw::Repository;
-$Git::Raw::Repository::VERSION = '0.40';
+$Git::Raw::Repository::VERSION = '0.41';
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Git::Raw::Repository - Git repository class
 
 =head1 VERSION
 
-version 0.40
+version 0.41
 
 =head1 SYNOPSIS
 
@@ -60,18 +60,10 @@ are:
 
 If true (default is false) create a bare repository.
 
-=item * "remote_name"
-
-The name to be given to the "origin" remote (default is "origin").
-
 =item * "checkout_branch"
 
 The name of the branch to checkout (default is to use the remote's HEAD).
 
-=item * "ignore_cert_errors"
-
-If true (default is false) ignore errors validating the remote host's certificate.
-
 =item * "disable_checkout"
 
 If true (default is false) files will not be checked out after the clone completes.
@@ -80,6 +72,16 @@ If true (default is false) files will not be checked out after the clone complet
 
 =over 8
 
+=item * "remote_create"
+
+Remote customization callback. If a non-default remote is required, i.e. a remote
+with a remote name other than 'origin', this callback should be used. The callback
+receives a L<Git::Raw::Repository> object, a string containing the default name
+for the remote, typically 'origin', and a string containing the URL of the remote.
+This callbacks should return a L<Git::Raw::Remote> object. The returned object and
+the the repository object passed to this callback is ephemeral. Do not take any
+references to it as it may be freed internally.
+
 =item * "credentials"
 
 The callback to be called any time authentication is required to connect to the
@@ -310,7 +312,7 @@ C<"type"> (setting C<"type"> to C<"soft"> or C<"hard"> has no effect).
 
 =back
 
-=head2 status( [$file, $file, ...] )
+=head2 status( \%opts, [$file, $file, ...] )
 
 Retrieve the status of files in the index and/or working directory. This functions
 returns a hash reference with an entry for each C<$file>, or all files if no file
@@ -323,9 +325,124 @@ If C<$file> has been renamed in either the index or worktree or both, C<$file>
 will also have a corresponding entry C<"index"> and/or C<"worktree">, containing
 the previous filename C<"old_file">.
 
+Valid fields for the C<%opts> hash are:
+
+=over 4
+
+=item * "flags"
+
+Flags for the status. Valid values include:
+
+=over 8
+
+=item * "include_untracked"
+
+Callbacks should be made on untracked files. These will only be made if the
+workdir files are included in the C<$show> option.
+
+=item * "include_ignored"
+
+Callbacks should be made on ignored files. These will only be made if the
+ignored files get callbacks.
+
+=item * "include_unmodified"
+
+Include even unmodified files.
+
+=item * "exclude_submodules"
+
+Submodules should be skipped. This only applies if there are no pending
+typechanges to the submodule (either from or to another type).
+
+=item * "recurse_untracked_dirs"
+
+All files in untracked directories should be included. Normally if an entire
+directory is new, then just the top-level directory is included (with a
+trailing slash on the entry name). This flag includes all of the individual
+files in the directory instead.
+
+=item * "disable_pathspec_match"
+
+Each C<$file> specified should be treated as a literal path, and not as a
+pathspec pattern.
+
+=item * "recurse_ignored_dirs"
+
+The contents of ignored directories should be included in the status. This is
+like doing C<git ls-files -o -i --exclude-standard> with core git.
+
+=item * "renames_head_to_index"
+
+Rename detection should be processed between the head and the index.
+
+=item * "renames_index_to_workdir"
+
+Rename detection should be run between the index and the working directory.
+
+=item * "sort_case_sensitively"
+
+Override the native case sensitivity for the file system and forces the output
+to be in case-sensitive order.
+
+=item * "sort_case_insensitively"
+
+Override the native case sensitivity for the file system and forces the output
+to be in case-insensitive order.
+
+=item * "renames_from_rewrites"
+
+Rename detection should include rewritten files.
+
+=item * "no_refresh"
+
+Bypass the default status behavior of doing a "soft" index reload (i.e.
+reloading the index data if the file on disk has been modified outside
+C<Git::Raw>).
+
+=item * "update_index"
+
+Refresh the stat cache in the index for files that are unchanged but have out
+of date stat information in the index. It will result in less work being done
+on subsequent calls to C<status>. This is mutually exclusive with the
+C<"no_refresh> option.
+
+=item * "include_unreadable"
+
+Include unreadable files.
+
+=item * "include_unreadable_as_untracked"
+
+Include unreadable files as untracked files.
+
+=back
+
+=item * "show"
+
+One of the following values (Defaults to C<index_and_worktree>):
+
+=over 8
+
+=item * "index_and_worktree"
+
+=item * "index"
+
+=item * "worktree"
+
+=back
+
+=back
+
 Example:
 
-    my $file_statuses = $repo -> status();
+    my $opts = {
+      'flags' => {
+        'include_untracked'        => 1,
+        'renames_head_to_index'    => 1,
+        'renames_index_to_workdir' => 1,
+      },
+      'show' => 'index_and_worktree'
+    };
+    my $file_statuses = $repo -> status($opts);
     while (my ($file, $status) = each %$file_statuses) {
       my $flags = $status -> {'flags'};
       print "File: $file: Status: ", join (' ', @$flags), "\n";
